@@ -7,5 +7,7 @@ from .help_text import FieldHelpTextMixin
 from .implicit_hidden import HiddenFieldMixin
 from .implicit_natural_date import DateFieldMixin, DateTimeFieldMixin, TimeFieldMixin
 from .null_choice import NullChoiceMixin
+from .password_field import PasswordFieldMixin
 from .related_field_ajax import RelatedFieldAJAXMixin
 from .render import DisplayMode, RenderMixin
+from .rtf_field import RTFFieldMixin
